import os
from importlib_resources import files

import cherrypy

from .controllers import Root

cherrypy.config.update(
    {
        'server.production': True,
        'server.socket_port': int(os.environ.get('PORT', 8080)),
        'server.socket_host': os.environ.get('SOCKET_HOST', '::'),
        'tools.encode.on': True,
        'tools.encode.encoding': 'utf-8',
        'tools.decode.on': True,
        'tools.sessions.on': True,
        'tools.trailing_slash.on': True,
        'tools.staticdir.root': os.fspath(files('jaraco.site')),
        'tools.proxy.on': True,
    }
)

config = {
    '/static': {
        'tools.staticdir.on': True,
        'tools.staticdir.dir': 'static',
        'tools.staticdir.content_types': dict(svg='image/svg+xml'),
    }
}

app = cherrypy.tree.mount(Root(), '/', config)



"""
Routines for installing, staging, and serving jaraco.com on Ubuntu.

To install on a clean Ubuntu Bionic box, simply run
fab bootstrap
"""

import itertools

from fabric import task
from jaraco.fabric import files
from jaraco.fabric import monkey

flatten = itertools.chain.from_iterable

host = 'spidey'
hosts = [host]

project = 'jaraco.site'
site = 'jaraco.com'
install_root = '/opt/jaraco.com'
python = 'python3.8'


@task(hosts=hosts)
def bootstrap(c):
    install_dependencies(c)
    install_env(c)
    install_service(c)
    update(c)
    configure_nginx(c)
    install_cert(c)
    install_scicomm(c)


@task(hosts=hosts)
def install_dependencies(c):
    # fop required by the resume endpoint
    c.sudo('apt install -y fop')
    # certbot for certificates
    c.sudo('apt-add-repository -y ppa:certbot/certbot')
    c.sudo('apt update -y')
    c.sudo('apt install -y python-certbot-nginx')

    c.sudo('apt install -y software-properties-common')
    c.sudo('add-apt-repository -y ppa:deadsnakes/ppa')
    c.sudo('apt update -y')
    c.sudo(f'apt install -y {python} {python}-venv')


def create_env(c, root):
    user = c.run('whoami').stdout.strip()
    c.sudo(f'mkdir -p {root}')
    c.sudo(f'chown {user} {root}')


@task(hosts=hosts)
@monkey.workaround_2090
def install_env(c):
    c.sudo(f'rm -R {install_root} || echo -n')
    create_env(c, install_root)
    c.run(f'{python} -m venv {install_root}')
    c.run(f'{install_root}/bin/python -m pip install -U pip')


@task(hosts=hosts)
def install_service(c):
    files.upload_template(
        c,
        f"ubuntu/{project}.service",
        "/etc/systemd/system",
        context=globals(),
    )
    c.sudo(f'systemctl enable {project}')


@task(hosts=hosts)
@monkey.workaround_2090
def update(c):
    install(c)
    install_scicomm(c)
    c.sudo(f'systemctl restart {project}')


def install(c):
    """
    Install project to environment at root.
    """
    c.run(f'git clone https://github.com/jaraco/{project} || echo -n')
    c.run(f'git -C {project} pull')
    c.run(f'{install_root}/bin/python -m pip install -U ./{project}')


@task(hosts=hosts)
@monkey.workaround_2090
def install_scicomm(c):
    root = '/opt/scicomm.pro'
    create_env(c, root)
    c.run(f'git clone https://github.com/jaraco/scicomm.pro {root} || echo -n')
    c.run(f'git -C {root} pull')


@task(hosts=hosts)
def remove_all(c):
    c.sudo(f'systemctl stop {project} || echo -n')
    c.sudo(f'rm /etc/systemd/system/{project}.service')
    c.sudo(f'rm -Rf {install_root}')


@task(hosts=hosts)
@monkey.workaround_2090
def configure_nginx(c):
    c.sudo('apt install -y nginx')
    source = "ubuntu/nginx config"
    target = f"/etc/nginx/sites-available/{site}"
    files.upload_template(c, src=source, dest=target)
    c.sudo(f'ln -sf ../sites-available/{site} /etc/nginx/sites-enabled/')
    c.sudo('service nginx restart')


@task(hosts=hosts)
def install_cert(c):
    cmd = [
        'certbot',
        '--agree-tos',
        '--email',
        'jaraco@jaraco.com',
        '--non-interactive',
        '--nginx',
        'certonly',
    ]
    sites = (
        'jaraco.com',
        'www.jaraco.com',
        'blog.jaraco.com',
        'www.recapturedocs.com',
        'scicomm.pro',
        'www.scicomm.pro',
    )
    cmd += list(flatten(['--domain', name] for name in sites))
    c.sudo(' '.join(cmd))



import os
import importlib

import cherrypy
from genshi.core import Stream
from genshi.output import encode, get_serializer
from genshi.template import Context, TemplateLoader


def init():
    return importlib.import_module('jaraco.site.run').app


class DefaultExtensionTemplateLoader(TemplateLoader):
    """
    A specialized template loader that will append a specific extension
    to each 'load' request if such extension is not present.
    """

    def __init__(self, *args, **kwargs):
        self.default_extension = kwargs.pop('extension', None)
        if self.default_extension:
            self.load = self.load_default
        return super(DefaultExtensionTemplateLoader, self).__init__(*args, **kwargs)

    def load_default(self, filename, *args, **kwargs):
        if not filename.endswith(self.default_extension):
            filename += self.default_extension
        return super(DefaultExtensionTemplateLoader, self).load(
            filename, *args, **kwargs
        )


loader = DefaultExtensionTemplateLoader(
    os.path.join(os.path.dirname(__file__), 'templates'),
    auto_reload=True,
    extension='.html',
)


# from the genshi tutorial
def output(
    filename, method='html', encoding='utf-8', content_type='text/html', **options
):
    """Decorator for exposed methods to specify what template they should use
    for rendering, and which serialization method and options should be
    applied.
    """

    def decorate(func):
        def wrapper(*args, **kwargs):
            cherrypy.response.headers['Content-Type'] = content_type
            cherrypy.thread_data.template = loader.load(filename)
            opt = options.copy()
            if method == 'html':
                opt.setdefault('doctype', 'html')
            serializer = get_serializer(method, **opt)
            stream = func(*args, **kwargs)
            if not isinstance(stream, Stream):
                return stream
            return encode(serializer(stream), method=serializer, encoding=encoding)

        return wrapper

    return decorate


def render(template_name=None, **kwargs):
    """Function to render the given data to the template specified via the
    ``@output`` decorator.
    """
    if isinstance(template_name, str):
        cherrypy.thread_data.template = loader.load(template_name)
    ctxt = Context(url=cherrypy.url)
    ctxt.push(kwargs)
    return cherrypy.thread_data.template.generate(ctxt)



import os
import subprocess

from requests_toolbelt import sessions
from lxml import etree


class Renderer:
    session = sessions.BaseUrlSession('https://dl.dropboxusercontent.com')
    url = '/s/sg48j6iuoc819jm/Jason R. Coombs resume.xml'

    def __init__(self, url=None):
        if url:
            self.url = url

    def get_transform_path(self, output_name):
        path_tmpl = 'resume-1.5.1/xsl/output/{output_name}.xsl'
        path = path_tmpl.format(**locals())
        here = os.path.dirname(__file__)
        return os.path.join(here, 'static', path)

    def html(self):
        transform_path = self.get_transform_path('us-html')
        transform = etree.XSLT(etree.parse(open(transform_path)))
        resp = self.session.get(self.url)
        src = etree.fromstring(resp.content)
        return str(transform(src))

    def pdf(self):
        "use subprocess and fop to render the output"
        cmd = ['fop', '-xml', '-', '-xsl', self.get_transform_path('us-letter'), '-']
        resp = self.session.get(self.url)
        proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stdin=subprocess.PIPE)
        stdout, stderr = proc.communicate(resp.content)
        return stdout
